(function() {"use strict";var __module = CC_EDITOR ? module : {exports:{}};var __filename = 'preview-scripts/assets/scripts/RhombicLayout.js';var __require = CC_EDITOR ? function (request) {return cc.require(request, require);} : function (request) {return cc.require(request, __filename);};function __define (exports, require, module) {"use strict";
cc._RF.push(module, '65105/NJklAuIda2hKqRyAx', 'RhombicLayout', __filename);
// scripts/RhombicLayout.ts

Object.defineProperty(exports, "__esModule", { value: true });
require("./utils/cc_vec_extension");
var game_events_1 = require("./game_events");
var executionOrder = cc._decorator.executionOrder;
var menu = cc._decorator.menu;
var _a = cc._decorator, ccclass = _a.ccclass, property = _a.property, executeInEditMode = _a.executeInEditMode;
var RhombicLayout = /** @class */ (function (_super) {
    __extends(RhombicLayout, _super);
    function RhombicLayout() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._edge_cells = 4;
        _this._cell_size = new cc.Vec2(220, 103);
        _this._cell_space = new cc.Vec2(5, 80);
        _this.half_cell_size = new cc.Vec2(0, 0);
        _this._total_cells = 16;
        _this._size = new cc.Vec2();
        _this.cell_centers = [];
        _this._y_axis_cell_count = 0;
        _this.border_points = [];
        return _this;
    }
    RhombicLayout_1 = RhombicLayout;
    Object.defineProperty(RhombicLayout.prototype, "edge_cells", {
        get: function () { return this._edge_cells; },
        set: function (value) {
            this._edge_cells = value;
            this.update_total_cells();
            this.update_y_axis_cell_count();
            this.update_relate_properties();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RhombicLayout.prototype, "cell_size", {
        get: function () { return this._cell_size; },
        set: function (value) {
            this._cell_size.set(value);
            this.update_half_cell_size();
            this.update_relate_properties();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RhombicLayout.prototype, "cell_space", {
        get: function () { return this._cell_space; },
        set: function (value) {
            this._cell_space.set(value);
            this.update_relate_properties();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RhombicLayout.prototype, "total_cells", {
        get: function () { return this._total_cells; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RhombicLayout.prototype, "graph", {
        get: function () { return this.graphics; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RhombicLayout.prototype, "y_axis_cell_count", {
        get: function () { return this._y_axis_cell_count; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RhombicLayout.prototype, "size", {
        get: function () { return this._size; },
        enumerable: true,
        configurable: true
    });
    /**
     * position the nodes with cell_centers
     * @param {cc.Node[]} nodes
     */
    RhombicLayout.prototype.layout = function (nodes) {
        if (nodes.length !== this.total_cells) {
            cc.error("cell_count not equal nodes count.cell count: " + this.total_cells + ", node count:" + nodes.length);
            return;
        }
        for (var i = 0; i < this.total_cells; ++i) {
            nodes[i].setPosition(this.cell_centers[i]);
        }
    };
    RhombicLayout.prototype.get_col_count = function (row) {
        return this.edge_cells - Math.abs(row - this.edge_cells + 1);
    };
    RhombicLayout.prototype.update_total_cells = function () {
        this._total_cells = this._edge_cells * this._edge_cells;
        cc.director.emit(game_events_1.game_events.total_cells_changed, this.total_cells);
    };
    /**
     * related with size_cells, cell_size, cell_space.
     */
    RhombicLayout.prototype.update_size = function () {
        var count = this.edge_cells;
        var space_count = count - 1;
        var w = count * this.cell_size.x + space_count * this.cell_space.x;
        var h = count * this.cell_size.y + space_count * this.cell_space.y;
        this._size.x = w;
        this._size.y = h;
        this.node.setContentSize(w, h);
    };
    RhombicLayout.prototype.update_cells_center = function () {
        //cc.log('update cell centers');
        var index = 0;
        var offset_rows = (this.y_axis_cell_count - 1) / 2.0;
        this.cell_centers = [];
        for (var row = offset_rows; row >= -offset_rows; --row) {
            //cc.log('row:', row);
            var y = (this.cell_size.y + this.cell_space.y) * row / 2;
            var cols = this.get_col_begin(row);
            var offset_cols = (cols - 1) / 2;
            //cc.log('cols:',cols);
            for (var col = -offset_cols; col <= offset_cols; ++col) {
                //cc.log('row:',row, 'col:',col);
                var x = (this.cell_size.x + this.cell_space.x) * col;
                this.cell_centers[index++] = new cc.Vec2(x, y);
            }
        }
        //cc.log('after update cell centers:', this.cell_centers);
    };
    RhombicLayout.prototype.get_col_begin = function (row) {
        return this.edge_cells - Math.abs(row);
    };
    RhombicLayout.prototype.update_y_axis_cell_count = function () {
        this._y_axis_cell_count = 2 * this.edge_cells - 1;
    };
    RhombicLayout.prototype.init_graphics = function () {
        var debug_node = this.node.get_or_add_child('_debug');
        debug_node.zIndex = 999;
        this.graphics = debug_node.get_or_add_component(cc.Graphics);
    };
    RhombicLayout.prototype.draw_gizmo = function () {
        this.graphics.clear();
        this.draw_border();
        this.graphics.stroke();
        this.draw_cells_border();
        this.graphics.stroke();
        this.draw_cell_centers();
        this.graphics.stroke();
        this.draw_cells_border();
    };
    RhombicLayout.prototype.draw_border = function () {
        this.graphics.strokeColor = cc.Color.RED;
        this.graphics.polyline(this.border_points);
    };
    RhombicLayout.prototype.draw_cell_centers = function () {
        this.graphics.strokeColor = cc.Color.GREEN;
        for (var i = 0; i < this.total_cells; ++i) {
            //cc.log('cell center:', this.cell_centers[i]);
            this.graphics.draw_cross(this.cell_centers[i], 10, cc.Color.GREEN);
        }
        this.graphics.stroke();
    };
    RhombicLayout.prototype.draw_cells_border = function () {
        this.graphics.strokeColor = cc.Color.GRAY;
        for (var _i = 0, _a = this.cells_border_points; _i < _a.length; _i++) {
            var points = _a[_i];
            this.graphics.polyline(points);
        }
    };
    /**
     * properties relate with edge_cells, cell_size, cell_space,
     */
    RhombicLayout.prototype.update_relate_properties = function () {
        this.update_size();
        this.update_border_points();
        this.update_cells_center();
        this.update_cells_border_points();
    };
    /**
     * call after update_size;
     */
    RhombicLayout.prototype.update_border_points = function () {
        var half_w = this.size.x / 2;
        var half_h = this.size.y / 2;
        this.border_points[0] = new cc.Vec2(0, half_h);
        this.border_points[1] = new cc.Vec2(half_w, 0);
        this.border_points[2] = new cc.Vec2(0, -half_h);
        this.border_points[3] = new cc.Vec2(-half_w, 0);
    };
    RhombicLayout.prototype.update_cells_border_points = function () {
        this.cells_border_points = [];
        for (var _i = 0, _a = this.cell_centers; _i < _a.length; _i++) {
            var center = _a[_i];
            var points = [];
            for (var i = 0; i < 4; ++i) {
                points[i] = RhombicLayout_1.get_corner_point(center, this.half_cell_size, i);
            }
            //cc.log('points:', points);
            this.cells_border_points.push(points);
        }
    };
    RhombicLayout.get_corner_point = function (center, half_size, index) {
        switch (index) {
            case 0:
                return center.add(new cc.Vec2(-half_size.x, 0));
            case 1:
                return center.add(new cc.Vec2(0, -half_size.y));
            case 2:
                return center.add(new cc.Vec2(half_size.x, 0));
            case 3:
                return center.add(new cc.Vec2(0, half_size.y));
        }
    };
    RhombicLayout.prototype.init_half_cell_size = function () {
        this.half_cell_size = new cc.Vec2(this.cell_size.x / 2, this.cell_size.y / 2);
    };
    RhombicLayout.prototype.update_half_cell_size = function () {
        this.half_cell_size.set_xy(this.cell_size.x / 2, this.cell_size.y / 2);
    };
    RhombicLayout.prototype.onLoad = function () {
        this.init_half_cell_size();
        this.update_y_axis_cell_count();
        this.update_size();
        this.init_graphics();
        this.update_border_points();
        this.update_cells_center();
        this.update_cells_border_points();
    };
    RhombicLayout.prototype.start = function () {
        cc.director.emit(game_events_1.game_events.total_cells_changed, this.total_cells);
    };
    RhombicLayout.prototype.onRestore = function () {
        this.onLoad();
    };
    RhombicLayout.prototype.update = function (dt) {
        this.draw_gizmo();
    };
    RhombicLayout.prototype.onDisable = function () {
        this.graphics.clear();
    };
    var RhombicLayout_1;
    __decorate([
        property
    ], RhombicLayout.prototype, "edge_cells", null);
    __decorate([
        property()
    ], RhombicLayout.prototype, "cell_size", null);
    __decorate([
        property({ tooltip: 'space between tow cell' })
    ], RhombicLayout.prototype, "cell_space", null);
    __decorate([
        property
    ], RhombicLayout.prototype, "total_cells", null);
    __decorate([
        property({ type: cc.Graphics })
    ], RhombicLayout.prototype, "graph", null);
    __decorate([
        property()
    ], RhombicLayout.prototype, "_edge_cells", void 0);
    __decorate([
        property(cc.Vec2)
    ], RhombicLayout.prototype, "_cell_space", void 0);
    __decorate([
        property()
    ], RhombicLayout.prototype, "_total_cells", void 0);
    RhombicLayout = RhombicLayout_1 = __decorate([
        ccclass(),
        executeInEditMode(),
        executionOrder(-1),
        menu('layer/rhombic layout')
    ], RhombicLayout);
    return RhombicLayout;
}(cc.Component));
exports.RhombicLayout = RhombicLayout;

cc._RF.pop();
        }
        if (CC_EDITOR) {
            __define(__module.exports, __require, __module);
        }
        else {
            cc.registerModuleFunc(__filename, function () {
                __define(__module.exports, __require, __module);
            });
        }
        })();
        //# sourceMappingURL=RhombicLayout.js.map
        